<?php

namespace App\HttpController\Street;

use App\Key\K;
use App\Model\Street\ConfigList;
use App\Model\Street\MapDownloadList;
use App\Model\Street\StreetErrorList;
use App\Model\Street\MapWorkList;
use App\Tools\System;
use EasySwoole\Component\TableManager;
use EasySwoole\ORM\DbManager;

//这个类主要是下载电子地图使用的
class Map extends Base
{
    public function index()
    {
        $outDate = [];
        $systemTable = TableManager::getInstance()->get(K::TABLE_SYSTEM_CONFIG);
        //如果有正在请求的，那么返回[];
        //后来在测试的过程中，总是有情况导致无法清空 K::MAP_IS_GETTING_ID 猜测有可能是线程重启的时候。导致的，所以我在线程重启的时候，执行一次清除
        if ($systemTable->get(K::MAP_IS_GETTING_ID)) return $this->jsonSuccess(['ids' => [], 'has_run' => 1]);
        $systemTable->set(K::MAP_IS_GETTING_ID, ['data' => 1, 'number' => 1]);
        $confirmTable = TableManager::getInstance()->get(K::TABLE_MAP_ID_CONFIRM_CACHE);
        //注意，这个ID，不能给的太大了，要不然子服务器无法处理大量的数据
        $id_limit = 1;
        foreach ($confirmTable as $v) {
            if (time() - $v['time'] > 600) {
                //超时10分钟，那么认为子服务器没有下载完成，注意，里面的数据也是一个数组，要和下面的对应起来
                $outDate[] = json_decode($v['data'], true);
                if (count($outDate) >= $id_limit) break;
            }
        }

        foreach ($outDate as $v) {
            //将这一部分的数据重新放到缓存中，进行处理
            $confirmTable->set($v['table_id'], ['time' => time(), 'data' => json_encode($v)]);
        }
        if ($outDate) {
            //如果有缓存的数据 ，先使用缓存的数据
            $systemTable->del(K::MAP_IS_GETTING_ID);
            return $this->jsonSuccess(['ids' => $outDate]);
        }
        System::log('cache table count: ' . $confirmTable->count());
        if ($confirmTable->count() > 200) {
            $systemTable->del(K::MAP_IS_GETTING_ID);
            //如果缓存中已经存在太多的数据，不会返回
            System::log('too many cache ids wait...');
            return $this->jsonSuccess(['ids' => []]);
        }
        $configModel = new ConfigList();
        $nextId = $configModel->getConfigData('next_map_id');
        $mapModel = new MapWorkList();
        if ($nextId) {
            $work = $mapModel->where('table_id', $nextId, '>')->order('table_id', 'asc')->limit(1)->get();
        } else {
            $work = $mapModel->where('table_id', 0, '>')->order('table_id', 'asc')->limit(1)->get();
        }
        if (!$work) {
            $systemTable->del(K::MAP_IS_GETTING_ID);
            System::log('no work to do');
            if (!$confirmTable->count()) {
                //此时，要返回 done 参数 ，表示服务器已经没有更多的数据了，要将最后一个ID上传上来
                return $this->jsonSuccess(['ids' => [], 'done' => 1]);
            } else {
                return $this->jsonSuccess(['ids' => []]);
            }
        }
        $work = $work->toArray();
        $confirmTable->set($work['table_id'], ['time' => time(), 'data' => json_encode($work)]);
        System::log('current worker next_id :' . $work['table_id']);
        $configModel->updateNextId('next_map_id', $work['table_id']);
        $systemTable->del(K::MAP_IS_GETTING_ID);
        return $this->jsonSuccess(['ids' => [$work]]);
    }

    public function confirm()
    {
        //客户端确认数据已经下载完成，那么将缓存中的数据要删除掉
        $data = $this->getRequestData();
        $table = TableManager::getInstance()->get(K::TABLE_MAP_ID_CONFIRM_CACHE);
        //这里返回的结果也是有下标的，要对应起来
        $workModel = new MapWorkList();
        foreach ($data['ids'] as $v) {
            $r = $table->del($v['table_id']);
            $workModel->where(['table_id' => $v['table_id']])->update(['done' => 1, 'ip' => $this->ip, 'done_time' => time()]);
        }
        echo 'confirm map sum',count($data['ids']),PHP_EOL;
        return $this->jsonSuccess($data);
    }
    //下载的子服务器将下载好的数据提交到这里来
    public function db()
    {
        $data = $this->getRequestData();
        $model = new MapDownloadList();
        DbManager::getInstance()->startTransaction('street');
        $del = [];
        foreach ($data['dbs'] as $v) {
            $has = $model->where(['id' => $v])->get();
            if ($has) {
                if ($has['is_done']) {
                    $model->where(['id' => $v])->update(['is_delete' => 1]);
                    $del[] = $v;
                }
            } else {
                $obj = $model::create(['id' => $v, 'ip' => $this->ip]);
                $obj->save();
            }
        }
        DbManager::getInstance()->commit('street');
        return $this->jsonSuccess(['del' => $del]);
    }

    //这个是获取当前有多少个IP服务器
    public function server()
    {
        $model = new MapDownloadList();
        switch ($this->request()->getMethod()) {
            case 'GET':
                $ips = $model->getIps();
                return $this->jsonSuccess($ips);
                break;
        }
    }

    public function download()
    {
        $model = new MapDownloadList();
        switch ($this->request()->getMethod()) {
            case 'POST':
            case 'GET':
                //这里的数据需要按IP来分类
                $data = $this->getRequestData();
                if ($data['ip']) {
                    $ip = [$data['ip']];
                } else {
                    $ip = $model->getIps();
                }
                $result = [];
                foreach ($ip as $v) {
                    $result[$v] = $model->order('id', 'ASC')->where(['is_done' => 0, 'ip' => $v])->limit(20)->all();
                    $result[$v] = array_map(function ($v) {
                        return $v->toArray();
                    }, $result[$v]);
                }
                return $this->jsonSuccess($result);
                break;
            case 'PUT':
                $data = $this->getRequestData();
                if (!$data['id']) return $this->jsonError('no id');
                $result = $model->groupSave($data['id'], 'edit_download', ['is_done' => 1, 'download_time' => time()]);
                if (!$result) return $this->jsonError($model->getError());
                return $this->jsonSuccess($result);
                break;
        }
    }

    public function error()
    {
        $data = $this->getRequestData();
        $model = new StreetErrorList();
        DbManager::getInstance()->startTransaction('street');
        foreach ($data as $v) {
            $has = $model->where(['id' => $v['id']])->get();
            if (!$has) {
                $data_str = is_array($v['data']) ? json_encode($v['data']) : $v['data'];
                $obj = $model::create(['id' => $v['id'], 'ip' => $this->ip, 'data' => $data_str, 'create_time_str' => date('Y-m-d H:i:s')]);
                $obj->save();
            }
        }
        DbManager::getInstance()->commit('street');
        return $this->jsonSuccess(['ok']);
    }
}
