<?php
namespace App\Tools;
class BMUtils
{
    /**
     * Get client real ip, Internal of Bigemap.
     */
    public static function getip()
    {
        static $ip = '';
        $ip = $_SERVER['REMOTE_ADDR'];
        if (isset($_SERVER['HTTP_CDN_SRC_IP'])) {
            $ip = $_SERVER['HTTP_CDN_SRC_IP'];
        } elseif (isset($_SERVER['HTTP_CLIENT_IP']) && preg_match('/^([0-9]{1,3}\.){3}[0-9]{1,3}$/', $_SERVER['HTTP_CLIENT_IP'])) {
            $ip = $_SERVER['HTTP_CLIENT_IP'];
        } elseif (isset($_SERVER['HTTP_X_FORWARDED_FOR']) and preg_match_all('#\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}#s', $_SERVER['HTTP_X_FORWARDED_FOR'], $matches)) {
            foreach ($matches[0] as $xip) {
                if (!preg_match('#^(10|172\.16|192\.168|127\.0)\.#', $xip)) {
                    $ip = $xip;
                    break;
                }
            }
        }
        return $ip;
    }

    /**
     * Zlib compress, Internal of Bigemap.
     */
    public static function z_compress($content)
    {
        $uncompressed_len = strlen($content);
        $uncompressed_len_str = self::bytes2str(self::int2bytes($uncompressed_len));

        $compressed = zlib_encode($content, ZLIB_ENCODING_DEFLATE);
        return $uncompressed_len_str . $compressed;
    }

    /**
     * Zlib uncompress, Internal of Bigemap.
     */
    public static function z_uncompress($content)
    {
        $uncompressed_len = self::bytes2int(self::str2bytes(substr($content, 0, 4)));
        $uncompressed = zlib_decode(substr($content, 4));

        if (strlen($uncompressed) != $uncompressed_len) {
            return "";
        }

        return $uncompressed;
    }

    /**
     * Convert int to bytes array.
     *
     * @param $littleEndian Little Endian, default true.
     */
    public static function int2bytes($number, $littleEndian = true)
    {
        $bytes = array();
        if ($littleEndian) {
            $bytes[0] = ($number & 0xFF);
            $bytes[1] = (($number >> 8) & 0xFF);
            $bytes[2] = (($number >> 16) & 0xFF);
            $bytes[3] = (($number >> 24) & 0xFF);
        } else {
            $bytes[0] = (($number >> 24) & 0xFF);
            $bytes[1] = (($number >> 16) & 0xFF);
            $bytes[2] = (($number >> 8) & 0xFF);
            $bytes[3] = ($number & 0xFF);
        }
        return $bytes;
    }

    /**
     * Convert bytes array to int.
     *
     * @param $littleEndian Little Endian, default true.
     */
    public static function bytes2int($bytes, $littleEndian = true)
    {
        if (count($bytes) != 4) return 0;

        if ($littleEndian) {
            $val = (($bytes[3] & 0xFF) << 24) | (($bytes[2] & 0xFF) << 16) |
                (($bytes[1] & 0xFF) << 8) | ($bytes[0] & 0xFF);
        } else {
            $val = (($bytes[0] & 0xFF) << 24) | (($bytes[1] & 0xFF) << 16) |
                (($bytes[2] & 0xFF) << 8) | ($bytes[3] & 0xFF);
        }
        return $val;
    }

    /**
     * Convert bytes array to string, will merge when give bytes arrays.
     *
     * @param $bytes_list Array of bytes or arrays.
     */
    public static function bytes2str(...$bytes_list)
    {
        $str = "";
        foreach ($bytes_list as $bytes) {
            foreach ($bytes as $b) {
                $str .= chr($b);
            }
        }
        return $str;
    }

    /**
     * Convert string to bytes array.
     *
     * @param string $str String.
     * @return array bytes array.
     */
    public static function str2bytes($str)
    {
        $bytes = array();
        $len = strlen($str);
        for ($i = 0; $i < $len; $i++) {
            $bytes[] = ord($str[$i]);
        }
        return $bytes;
    }

    /**
     * Base64 encode in url safe mode.
     */
    public static function urlsafe_b64encode($string)
    {
        $data = base64_encode($string);
        $data = str_replace(array('+', '/', '='), array('-', '_', ''), $data);
        return $data;
    }

    /**
     * Base64 decode in url safe mode.
     */
    public static function urlsafe_b64decode($string)
    {
        $data = str_replace(array('-', '_'), array('+', '/'), $string);
        $mod4 = strlen($data) % 4;
        if ($mod4) {
            $data .= substr('====', $mod4);
        }
        return base64_decode($data);
    }
}
